@@ -69,17 +69,20 @@ freepops.MODULES_MAP["yahoo.ie"]       	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.it"]       	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.ca"] 	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.co.in"] 	= {name="yahoo.lua"}
+freepops.MODULES_MAP["yahoo.co.id"] 	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.com.tw"] 	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.co.uk"] 	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.com.cn"] 	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.com.br"] 	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.com.hk"] 	= {name="yahoo.lua"}
+freepops.MODULES_MAP["yahoo.com.au"] 	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.es"] 	= {name="yahoo.lua"}
 freepops.MODULES_MAP["yahoo.de"]        = {name="yahoo.lua"} 
 freepops.MODULES_MAP["yahoo.dk"]        = {name="yahoo.lua"} 
 freepops.MODULES_MAP["yahoo.fr"]        = {name="yahoo.lua"} 
 freepops.MODULES_MAP["rocketmail.com"]  = {name="yahoo.lua"}
 freepops.MODULES_MAP["talk21.com"]  = {name="yahoo.lua"}
+freepops.MODULES_MAP["btinternet.com"]  = {name="yahoo.lua"}
  
 
 -- hotmail
@@ -124,6 +127,25 @@ freepops.MODULES_MAP["aim.com"]		= {name="aol.lua"}
 -- netscape
 freepops.MODULES_MAP["netscape.net"]    = {name="netscape.lua"}
 
+-- abv
+freepops.MODULES_MAP["abv.bg"]   	= {name="abv.lua"}
+freepops.MODULES_MAP["gyuvetch.bg"]    	= {name="abv.lua"}
+freepops.MODULES_MAP["gbg.bg"]    	= {name="abv.lua"}
+
+-- Orange
+freepops.MODULES_MAP["fsmail.net"]   	= {name="orange.lua"}
+freepops.MODULES_MAP["wanadoo.nl"]    	= {name="orange.lua"}
+freepops.MODULES_MAP["orange.nl"]    	= {name="orange.lua"}
+freepops.MODULES_MAP["orange.co.uk"]   	= {name="orange.lua"}
+freepops.MODULES_MAP["bedrijfsnaam.nl"] = {name="orange.lua"}
+
+-- softhome
+freepops.MODULES_MAP["softhome.net"]    = {name="softhome.lua"}
+
+-- excite/myway
+freepops.MODULES_MAP["myway.com"]	= {name="excite.lua"}
+freepops.MODULES_MAP["excite.com"]	= {name="excite.lua"}
+
 -- squirrelmail
 freepops.MODULES_MAP["mydom.com"]	= {name="squirrelmail.lua"}
 
@@ -439,7 +461,6 @@ freepops.MODULES_MAP["xy.com.tw"]		= {name="mailcom.lua"}
 freepops.MODULES_MAP["yankeeman.com"]		= {name="mailcom.lua"}
 freepops.MODULES_MAP["yyhmail.com"]		= {name="mailcom.lua"}
 freepops.MODULES_MAP["verizonmail.com"]		= {name="mailcom.lua"}
-freepops.MODULES_MAP["lycos.com" ]		= {name="mailcom.lua"}
 freepops.MODULES_MAP["unforgettable.com" ]	= {name="mailcom.lua"}
 freepops.MODULES_MAP["mail.org" ]		= {name="mailcom.lua"}
 freepops.MODULES_MAP["italymail.com" ]		= {name="mailcom.lua"}
@@ -680,6 +701,11 @@ freepops.MODULES_MAP["linuxgazette.net"] 	= {
 --
 freepops.MODULES_MAP["updater"] = {name="updater.lua"}
 
+-- Monitor
+--
+freepops.MODULES_MAP["monitor"] = {name="monitor.lua"}
+
+
 -- -------------------------------------------------------------------------- --
 -- 2) Policy
 --
