@@ -59,9 +59,9 @@
 -- <B>ssl_init_stuff()</B> : some stuff for SSL<BR/>
 --<BR/>
 
-MODULE_VERSION = "0.1.0"
+MODULE_VERSION = "0.1.1"
 MODULE_NAME = "browser.browser"
-MODULE_REQUIRE_VERSION = "0.2.0"
+MODULE_REQUIRE_VERSION = "0.2.6"
 MODULE_LICENSE = "GNU/GPL"
 MODULE_URL = "http://www.freepops.org/download.php?module=browser.browser.lua"
 MODULE_HOMEPAGE = "http://www.freepops.org/"
@@ -194,7 +194,13 @@ end
 -- adds dirname(u.path) .. / .. location if needed
 function Hidden.adjust_path(l,u,location)
 	local function clean_2_slash(s)
-		return (string.gsub(s,"//","/"))
+                -- Change made by Kevin Edwards
+                --
+ 		local subst = 1
+ 		while subst > 0 do
+ 			s,subst = string.gsub(s,"//","/")
+ 		end
+ 		return s
 	end
 	local function dirname(path)
 		local base = ""
@@ -211,7 +217,7 @@ function Hidden.adjust_path(l,u,location)
 				function(s)table.insert(rc,s)
 			end)
 			-- delete last element
-			table.remove(rc,table.getn(rc))
+			table.remove(rc,#rc)
 			base = "/" .. table.concat(rc,"/") .. "/"
 		end
 		return clean_2_slash(base)
@@ -316,7 +322,7 @@ function Hidden.parse_header(self,gl_h,url)
                 Hidden.is_https_proxy_tunnel(self, url, ret) then
                 local gl_h1 = {} -- to not lose the real header
                 local end_of_1xx = false
-                for i=1,table.getn(gl_h) do
+                for i=1,#gl_h do
                        if end_of_1xx then
                              table.insert(gl_h1,gl_h[i])
                        end
@@ -732,11 +738,11 @@ function new(override_useragent)
 		cookies = {},
 		referrer = false, --nil will break the metatable check
 		curl = false,
-		proxy = os.getenv("LUA_HTTP_PROXY"),
-		proxyauth = os.getenv("LUA_HTTP_PROXYAUTH"),
+		proxy = os.getenv("FREEPOPSLUA_HTTP_PROXY"),
+		proxyauth = os.getenv("FREEPOPSLUA_HTTP_PROXYAUTH"),
 		useragent = override_useragent or 
-		  os.getenv("LUA_HTTP_USERAGENT"),
-		fpat = os.getenv("LUA_FORCE_PROXY_AUTH_TYPE"),
+		  os.getenv("FREEPOPSLUA_HTTP_USERAGENT"),
+		fpat = os.getenv("FREEPOPSLUA_FORCE_PROXY_AUTH_TYPE"),
 		followRefreshHeader = false,
 	}
 
