@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: dictionary.c,v 1.3 2007/01/13 22:36:23 gareuselesinge Exp $
+ * $Id: dictionary.c,v 1.4 2007/10/27 13:14:14 gareuselesinge Exp $
  * This file is part of liberopops (http://liberopops.sf.net)                 *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -26,10 +26,14 @@
 struct couple_t 
 	{
 	char* key;
-	void*data;
+	void* data;
 	};
 
-HIDDEN pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
+struct dictionary_t 
+	{
+	list_t *head;
+	pthread_mutex_t lock;
+	};
 
 /******************************************************************************/
 HIDDEN struct couple_t *new_couple(const char* k,void *d)
@@ -59,53 +63,95 @@ return !strcmp(k,c->key);
 }
 
 /******************************************************************************/
-void* dictionary_find(struct dictionary_t *d,const char* key)
-{
-list_t *l;
-pthread_mutex_lock(&lock);
-l = list_find(d->head,(void*)key,equal);
-pthread_mutex_unlock(&lock);
-if(l != NULL)
-	return ((struct couple_t*)(l->data))->data;
+struct dictionary_t *dictionary_create(){
+struct dictionary_t * tmp;
 
-return NULL;
+tmp = malloc(sizeof(struct dictionary_t));
+MALLOC_CHECK(tmp);
+
+tmp->head = NULL;
+pthread_mutex_init(&(tmp->lock),NULL);
+
+return tmp;
 }
 
-int dictionary_remove(struct dictionary_t *d,const char* key)
+
+int dictionary_find(struct dictionary_t *d,const char* key,
+			void **res, int (*op)(void *))
 {
 list_t *l;
-pthread_mutex_lock(&lock);
+int rc=0;
+void * data = NULL;
+
+pthread_mutex_lock(&(d->lock));
 l = list_find(d->head,(void*)key,equal);
-if(l != NULL)
-	{
-	delete_couple((struct couple_t*)l->data);
-	d->head=list_remove(d->head,l);
-	pthread_mutex_unlock(&lock);
-	return 0;
-	}
-else
-	{
-	pthread_mutex_unlock(&lock);
-	return 1;
-	}
+if (l != NULL) {
+	data = ((struct couple_t*)(l->data))->data;
+	if (op != NULL) rc = op(data);
+}
+pthread_mutex_unlock(&(d->lock));
+
+if (res != NULL) {
+	*res = data;
 }
 
-int dictionary_add(struct dictionary_t *d,const char* key,void *data)
+return rc;
+}
+
+int dictionary_remove(
+	struct dictionary_t *d, const char* key,
+	int (*op)(void *), void(*freedata)(void*))
 {
 list_t *l;
-pthread_mutex_lock(&lock);	
+int rc = 0;
+void * data = NULL;
+
+pthread_mutex_lock(&(d->lock));
 l = list_find(d->head,(void*)key,equal);
-if(l == NULL)
-	{
+if(l != NULL) {
+	data = ((struct couple_t*)(l->data))->data;
+	if (op != NULL) {
+		rc = op(data);
+	}	
+	if (rc == 0){
+		if (freedata != NULL) freedata(data);
+		delete_couple((struct couple_t*)l->data);
+		d->head=list_remove(d->head,l);
+	}
+} else	{
+	rc = 1;
+}
+pthread_mutex_unlock(&(d->lock));
+
+return rc;
+}
+
+int dictionary_add(
+	struct dictionary_t *d,const char* key,
+	void *data, int (*op)(void *), void(*freedata)(void*))
+{
+list_t *l;
+int rc = 1;
+void *olddata = NULL;
+
+pthread_mutex_lock(&(d->lock));	
+l = list_find(d->head,(void*)key,equal);
+if(l == NULL) {
 	d->head = list_add(d->head,new_couple(key,data));
-	pthread_mutex_unlock(&lock);
-	}
-else
-	{
-	pthread_mutex_unlock(&lock);
-	return 1;
+} else {
+	olddata = ((struct couple_t*)(l->data))->data;
+	if (op != NULL) {
+		rc = op(olddata);
+	} 
+	if (rc == 0 && freedata != NULL) {
+		freedata(olddata);
+		((struct couple_t*)(l->data))->data = data;
+	} else {
+		rc = 2;
 	}
+}
+pthread_mutex_unlock(&(d->lock));
 
-return 0;
+return rc;
 }
 
