@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: popstate.c,v 1.8 2007/06/22 15:10:38 gareuselesinge Exp $
+ * $Id: popstate.c,v 1.9 2007/10/23 15:08:54 gareuselesinge Exp $
  * This file is part of liberopops (http://liberopops.sf.net)                 *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -65,7 +65,7 @@ void delete_mailmessage(struct mail_msg_t* m);
 struct mail_msg_t* new_mailmessage();
 
 /******************************************************************************/
-struct popstate_t *new_popstate_t(void *newp(void))
+struct popstate_t *new_popstate_t(void)
 {
 struct popstate_t *tmp;
 
