@@ -1,7 +1,7 @@
 -- ************************************************************************** --
 --  FreePOPs @gmail.com webmail interface
 --  
---  $Id: gmail.lua,v 1.44 2007/01/14 16:49:36 gareuselesinge Exp $
+--  $Id: gmail.lua,v 1.45 2007/10/21 18:07:23 gareuselesinge Exp $
 --  
 --  Released under the GNU/GPL license
 --  Written by Rami Kattan <rkattan at gmail (single dot) com>
@@ -13,7 +13,7 @@
 -- ************************************************************************** --
 
 -- these are used in the init function
-PLUGIN_VERSION = "0.0.45"
+PLUGIN_VERSION = "0.0.46"
 PLUGIN_NAME    = "GMail.com"
 PLUGIN_REQUIRE_VERSION = "0.2.0"
 PLUGIN_LICENSE = "GNU/GPL"
@@ -698,7 +698,7 @@ function stat(pstate)
 				s, strMessageListRegExp, iPos1)
 		end
 
-		local n = table.getn(MessageList)
+		local n = #MessageList
 
 		if n == 0 then
 			return true,nil
