@@ -3,11 +3,13 @@
 --  
 --  Released under the GNU/GPL license
 --  Written by Russell Schwager <russell822@yahoo.com>
+--  Patched by cdmackie (2007-09-11). 
+--    Fixed: lite login, live delete, corrupt messages, stuck at 1st message, classic delete more than 1 message
 -- ************************************************************************** --
 
 -- Globals
 --
-PLUGIN_VERSION = "0.1.81c"
+PLUGIN_VERSION = "0.1.88a"
 PLUGIN_NAME = "hotmail.com"
 PLUGIN_REQUIRE_VERSION = "0.2.0"
 PLUGIN_LICENSE = "GNU/GPL"
@@ -50,6 +52,12 @@ pulls as unread.  If the value is 1, the behavior is turned on.]]
 Parameter is used to force the plugin to only download a maximum number of messages. ]]
 		}	
 	},
+	{name = "domain", description = {
+		en = [[
+Parameter is used to override the domain in the email address.  This is used so that users don't
+need to add a mapping to config.lua for a hosted hotmail account. ]]
+		}	
+	},
 }
 PLUGIN_DESCRIPTIONS = {
 	it=[[
@@ -115,9 +123,12 @@ local globals = {
   -- Pattern to detect if we are using the live or classic version
   --
   strLiveCheckPattern = '(TodayLight%.aspx)',
+  strLiveCheckPattern2 = '(todayPageOptOut: true)',
   strClassicCheckPattern = '(Windows Live Mail was not able to sign into your account at this time)',
   strLiveMainPagePattern = '<frame.-name="main" src="([^"]+)"',
-  strLiveLightPagePattern = 'href="(StylesheetTodayLight)',
+  -- cdmackie: some version do not have this anymore, so use something common
+  -- strLiveLightPagePattern = 'href="(StylesheetTodayLight)',
+  strLiveLightPagePattern = '"MailClassic"',  
 
   -- Get the crumb value that is needed for every command
   --
@@ -127,7 +138,7 @@ local globals = {
 
   -- MSN Inbox Folder Id
   --
-  strPatMSNInboxId = "HMFO%('(%d+)'%)[^>]+><img src=.http://[^/]+/i.p.folder.inbox.gif",
+  strPatMSNInboxId = "HMFO%('(%d+)'%)[^>]+><img src=.http://[^/]+/i%.p%.folder%.inbox%.gif",
 
   -- Image server pattern
   --
@@ -144,10 +155,16 @@ local globals = {
   strFolderPattern = '<a href="[^"]+curmbox=([^&]+)&[^"]+" >', 
   strFolderLivePattern = '%("([^"]+)","',
   strFolderLiveInboxPattern = 'sysfldrinbox".-"([^"]+)"',
-  strFolderLiveLightInboxPattern = 'href="InboxLight%.aspx%?FolderID=([^&]+)&n=[^"]+"[^>]+><img src=".-i_inbox.gif"',
+  strFolderLiveLightInboxPattern = 'href="InboxLight%.aspx%?(FolderID=[^&]+[^"]+)"[^>]+><img src=".-i_inbox.gif"',
+  strFolderLiveLightFolderIdPattern = 'FolderID=([^&]+)&[.]*',
+  strFolderLiveLightNPattern = '&n=([^&]+)[.]*',
+  
   strFolderLiveLightTrashPattern = 'i_trash%.gif" border="0" alt=""/></td>.-<td class="dManageFoldersFolderNameCol"><a href="InboxLight%.aspx%?FolderID=([^&]+)&',
+  strFolderLiveLightTrash2Pattern = 'href="InboxLight%.aspx%?FolderID=([^&]+)&[^"]+"[^>]+><img src="[^"]+" class="i_trash"',
   strFolderLiveLightJunkPattern = 'i_junkfolder%.gif" border="0" alt=""/></td>.-<td class="dManageFoldersFolderNameCol"><a href="InboxLight%.aspx%?FolderID=([^&]+)&',
-  strFolderLiveLightPattern = 'href="InboxLight%.aspx%?FolderID=([^&]+)&n=[^"]+" title="',  
+  strFolderLiveLightJunk2Pattern = 'href="InboxLight%.aspx%?FolderID=([^&]+)&[^"]+"[^>]+><img src="[^"]+" class="i_junkfolder"',
+  strFolderLiveLightPattern = 'href="InboxLight%.aspx%?FolderID=([^&]+&n=[^"]+)" title="',  
+  strFolderLiveLightManageFoldersPattern = 'href="ManageFoldersLight%.aspx%?n=([^"]+)"',
 
   -- Pattern to determine if we have no messages
   --
@@ -157,7 +174,7 @@ local globals = {
   --
   strMsgListCntPattern = "<td width=100. align=center>([^<]+)</td><td align=right nowrap>",
   strMsgListCntPattern2 = "([%d]+) [MmNnBbVv][eai]",
-  strMsgListLiveLightCntPattern = '<div class="dItemListHeaderMsgInfo">.- (%d+)</div>',
+  strMsgListLiveLightCntPattern = '<div class="dItemListHeaderMsgInfo".->.-(%d+).-</div>',
 
   -- Used by Stat to pull out the message ID and the size
   --
@@ -168,6 +185,7 @@ local globals = {
   -- Pattern used by Stat to get the next page in the list of messages
   --
   strMsgListNextPagePattern = '(nextpg%.gif" border=0></a>)',
+  strMsgListNextPagePatLiveLight = '<a href="([^"]+)"[^>]*><img src="[^_]*_nextpage.gif"',
 
   -- Pattern used to detect a bad STAT page.
   --
@@ -206,9 +224,10 @@ local globals = {
   strCmdDeletePost = "curmbox=%s&_HMaction=delete&wo=&SMMF=0", -- &<MSGID>=on
   strCmdDeleteLive = "http://%s/mail/mail.fpp?cnmn=Microsoft.Msn.Hotmail.MailBox.MoveMessages&ptid=0&a=%s&au=%s", 
   strCmdDeletePostLiveOld = 'cn=Microsoft.Msn.Hotmail.MailBox&mn=MoveMessages&d="%s","%s",[%s],[{"%%5C%%7C%%5C%%7C%%5C%%7C0%%5C%%7C%%5C%%7C%%5C%%7C00000000-0000-0000-0000-000000000001%%5C%%7C632901424233870000",{2,"00000000-0000-0000-0000-000000000000",0}}],null,null,0,false,Date&v=1',
-  strCmdDeletePostLive = 'cn=Microsoft.Msn.Hotmail.MailBox&mn=MoveMessages&d="%s","%s",[%s],[{"%%5C%%7C%%5C%%7C%%5C%%7C0%%5C%%7C%%5C%%7C%%5C%%7C%%5C%%7C00000000-0000-0000-0000-000000000001%%5C%%7C632750213035330000",null}],null,null,0,false,Date,false,true&v=1&mt=%s',
-  strCmdDeleteLiveLight = "http://%s/mail/InboxLight.aspx?FolderID=%s&", 
-  strCmdDeletePostLiveLight = "__VIEWSTATE=&mt=%s&query=&MoveMessageSelector=%s&ToolbarActionItem=MoveMessageSelector&", -- SelectedMessages=%s",
+  -- strCmdDeletePostLive = 'cn=Microsoft.Msn.Hotmail.MailBox&mn=MoveMessages&d="%s","%s",[%s],[{"%%5C%%7C%%5C%%7C%%5C%%7C0%%5C%%7C%%5C%%7C%%5C%%7C%%5C%%7C00000000-0000-0000-0000-000000000001%%5C%%7C632750213035330000",null}],null,null,0,false,Date,false,true&v=1&mt=%s',
+  strCmdDeletePostLive = 'cn=Microsoft.Msn.Hotmail.MailBox&mn=MoveMessages&d="%s","%s",[%s],[{"0%%5C%%7C0%%5C%%7C8C9BDFF65883200%%5C%%7C00000000-0000-0000-0000-000000000001",null}],null,null,0,false,Date,false,true&v=1&mt=%s',
+  strCmdDeleteLiveLight = "http://%s/mail/InboxLight.aspx?FolderID=%s&",
+  strCmdDeletePostLiveLight = "__VIEWSTATE=&mt=%s&MoveMessageSelector=%s&ToolbarActionItem=MoveMessageSelector&", -- SelectedMessages=%s",
   strCmdMsgView = "http://%s/cgi-bin/getmsg?msg=%s&imgsafe=y&curmbox=%s&a=%s",
   strCmdMsgViewRaw = "&raw=0",
   strCmdMsgViewLive = "http://%s/mail/GetMessageSource.aspx?msgid=%s&gs=true",
@@ -216,16 +235,16 @@ local globals = {
   strCmdLogout = "http://%s/cgi-bin/logout",
   strCmdLogoutLive = "http://%s/mail/logout.aspx",
   strCmdFolders = "http://%s/cgi-bin/folders?&curmbox=F000000001&a=%s",
-  strCmdFoldersLiveLight = "http://%s/mail/ManageFoldersLight.aspx",
+  strCmdFoldersLiveLight = "http://%s/mail/ManageFoldersLight.aspx?n=%s",
   strCmdMsgUnreadLive = "http://%s/mail/mail.fpp?cnmn=Microsoft.Msn.Hotmail.MailBox.MarkMessages&ptid=0&a=", 
   strCmdMsgUnreadLivePost = "cn=Microsoft.Msn.Hotmail.MailBox&mn=MarkMessages&d=false,[%s]",
   strCmdEmptyTrashLive = "http://%s/mail/mail.fpp?cnmn=Microsoft.Msn.Hotmail.MailBox.EmptyFolder&ptid=0&a=&au=%s", 
-  strCmdEmptyTrashLivePost = "cn=Microsoft.Msn.Hotmail.MailBox&mn=EmptyFolder&d=%s,0",
+  strCmdEmptyTrashLivePost = "cn=Microsoft.Msn.Hotmail.MailBox&mn=EmptyFolder&d=%s,1&v=1&mt=%s",
   strCmdEmptyTrashLiveLight = "http://%s/mail/InboxLight.aspx?EmptyFolder=True&FolderID=%s&", 
   strCmdEmptyTrashLiveLightPost = "__VIEWSTATE=&mt=%s&query=&MoveMessageSelector=&ToolbarActionItem=&InfoPaneActionItem=EmptyFolderConfirmYes",
   strCmdMsgReadLive = "http://%s/mail/mail.fpp?cnmn=Microsoft.Msn.Hotmail.MailBox.MarkMessages&ptid=0&a=&au=%s", 
   strCmdMsgReadLivePost = "cn=Microsoft.Msn.Hotmail.MailBox&mn=MarkMessages&d=true,[%s]&v=1&mt=%s",
-  strCmdMsgReadLiveLight = "http://%s/mail/ReadMessageLight.aspx?Aux=&FolderID=%s&InboxSortAscending=False&InboxSortBy=Date&ReadMessageId=%s",
+  strCmdMsgReadLiveLight = "http://%s/mail/ReadMessageLight.aspx?AllowUnsafe=True&Aux=&FolderID=%s&InboxSortAscending=False&InboxSortBy=Date&ReadMessageId=%s",
 }
 
 -- ************************************************************************** --
@@ -475,9 +494,8 @@ function loginHotmail()
 
   -- Check to see if we are using the new interface and are redirecting.
   --
-  str = string.match(body, globals.strLiveCheckPattern)
   local folderBody = body
-  if str ~= nil then
+  if string.match(body, globals.strLiveCheckPattern) or string.match(body, globals.strLiveCheckPattern2) then
     log.dbg("Hotmail: Detected LIVE version.") 
     str = string.format(globals.strCmdBrowserIgnoreLive, browser:wherearewe())
     body, err = browser:get_uri(str)
@@ -502,9 +520,14 @@ function loginHotmail()
     -- One or two more redirects
     --  
     local oldurl = url
+    local oldbody = body
     url = string.match(body, globals.strLoginDoneReloadToReloadPage)
     if url ~= nil then
       body, err = browser:get_uri(url)
+      if body == nil then
+       	-- cdmackie: switch back to old body because we need to first process img in strLoginDoneReloadToHMHome4
+       	body = oldbody
+      end
     end
     url = string.match(body, globals.strLoginDoneReloadToHMHome1)
     if url == nil then
@@ -628,7 +651,13 @@ function loginHotmail()
       internalState.strTrashId = str
       log.dbg("Hotmail - trash folder id: " .. str)
     else
-      log.error_print("Unable to detect the folder id for the trash folder.  Deletion may fail.")
+      str = string.match(body, globals.strFolderLiveLightTrash2Pattern) 
+      if str ~= nil then
+        internalState.strTrashId = str
+        log.dbg("Hotmail - trash folder id: " .. str)
+      else
+        log.error_print("Unable to detect the folder id for the trash folder.  Deletion may fail.")
+      end
     end
 
     str = string.match(body, globals.strPatLiveJunkId) 
@@ -636,12 +665,18 @@ function loginHotmail()
       internalState.strJunkId = str
       log.dbg("Hotmail - junk folder id: " .. str)
     else
-      log.error_print("Unable to detect the folder id for the junk folder.  Deletion may fail.")
+      str = string.match(body, globals.strFolderLiveLightJunk2Pattern) 
+      if str ~= nil then
+        internalState.strJunkId = str
+        log.dbg("Hotmail - junk folder id: " .. str)
+      else
+        log.error_print("Unable to detect the folder id for the junk folder.  Deletion may fail.")
+      end
     end
 
     if (internalState.strMBoxName == "Inbox") then
       str = inboxId
-    elseif (internalState.strMBoxName == "Junk") then
+    elseif (internalState.strMBoxName == "Junk" or internalState.strMBoxName == "Junk E%-Mail") then
       str = internalState.strJunkId
     else
       str = getFolderId(inboxId)
@@ -655,9 +690,17 @@ function loginHotmail()
       log.dbg("Hotmail - Using folder (" .. internalState.strMBox .. ")")
     end
   elseif (internalState.bLiveGUI == true and internalState.bLiveLightGUI == true) then 
-    local url = string.format(globals.strCmdFoldersLiveLight, internalState.strMailServer)
+    -- cdmackie: Live Light has an "n" value in the folders list that we need to get
+    local n = string.match(body, globals.strFolderLiveLightManageFoldersPattern)
+    local url = string.format(globals.strCmdFoldersLiveLight, internalState.strMailServer, n)    
+    
     body, err = browser:get_uri(url)
-    local inboxId = string.match(body, globals.strFolderLiveLightInboxPattern)
+    
+    -- cdmackie: we then extract the querystring and apend the InboxId with the N value
+    local inboxQueryString = string.match(body, globals.strFolderLiveLightInboxPattern)
+    local inboxId = string.match(inboxQueryString, globals.strFolderLiveLightFolderIdPattern)
+    local inboxN = string.match(inboxQueryString, globals.strFolderLiveLightNPattern)
+    inboxId = inboxId .. "&n=" .. inboxN
 
     if (internalState.strMBoxName == "Inbox") then
       str = inboxId
@@ -797,17 +840,36 @@ function downloadMsg(pstate, msg, nLines, data)
   -- Handle whatever is left in the buffer
   --
   local body = cbInfo.strBuffer
-  if (string.len(body) > 0 and cbInfo.nLinesReceived == -2) then
+  if (string.len(body) > 0 and cbInfo.nLinesRequested == -2) then
     log.raw("Message: " .. cbInfo.cb_uidl .. ", left over buffer being processed: " .. body)
     body = cleanupBody(body, cbInfo)
     
+    -- cdmackie: rather than test for </pre> to determine end, just set it seen
+    -- as we've reached end of buffer..and hotmail doesn't always send "</pre>"
+    cbInfo.bEndReached = true
+    cbInfo.nLinesReceived = -1;
+    -- apply same fixup from cleanupbody to remove dead tags
+    if (string.len(body) > 6) then
+      local idx = string.find(string.sub(body, -6), "([&<])")
+      if (idx ~= nil) then
+        idx = idx - 1
+        cbInfo.strBuffer = string.sub(body, -6 + idx)
+        local len = string.len(body) - 6 + idx
+        body = string.sub(body, 1, len)
+      end
+    end
+    -- make sure we end in a crlf
+    if (string.sub(body, -2, -1) ~= "\r\n") then
+      body = body .. "\r\n"
+    end
+    
     if (cbInfo.bEndReached == false) then
-      log.data("Forcing a CRLF to end the message as it isn't clear the message is ended properly")
+      log.dbg("Forcing a CRLF to end the message as it isn't clear the message is ended properly")
       popserver_callback("\r\n\0", data)
     end
     body = cbInfo.strHack:dothack(body) .. "\0"
     popserver_callback(body, data)
-  elseif (cbInfo.bEndReached == false and cbInfo.nLinesReceived == -2) then
+  elseif (cbInfo.bEndReached == false and cbInfo.nLinesRequested == -2) then
       popserver_callback("\r\n\0", data)
   end
 
@@ -822,9 +884,9 @@ function downloadMsg(pstate, msg, nLines, data)
     local post = string.format(globals.strCmdMsgReadLivePost, uidl, internalState.strMT)
     browser:post_uri(url, post)
   elseif internalState.bMarkMsgAsUnread == false and internalState.bLiveGUI and internalState.bLiveLightGUI then
-    log.raw("Message: " .. cbInfo.cb_uidl .. ", Marking message as being done.")
+    log.raw("Message: " .. cbInfo.cb_uidl .. ", Marking message as read.")
     url = string.format(globals.strCmdMsgReadLiveLight, internalState.strMailServer, internalState.strMBox, uidl)
-    browser:get_head(markReadUrl)
+    browser:get_head(url)
   elseif internalState.bMarkMsgAsUnread == true and internalState.bLiveGUI == true then
     log.raw("Message: " .. cbInfo.cb_uidl .. ", Marking message as unread.")
     url = string.format(globals.strCmdMsgUnreadLive, internalState.strMailServer)
@@ -979,10 +1041,12 @@ function cleanupBody(body, cbInfo)
 
   -- Did we reach the end of the message
   --
-  if (string.find(body, "(</pre>)") ~= nil) then
-    cbInfo.nLinesReceived = -1;
-    cbInfo.bEndReached = true 
-  end
+  -- cdmackie: test for the end after we've processed the buffers instead
+  -- GetMessageSource sometimes returns only "</" instead of "</pre>"
+  --if (string.find(body, "(</pre>)") ~= nil) then
+  --  cbInfo.nLinesReceived = -1;
+  --  cbInfo.bEndReached = true 
+  --end
 
   -- The only parts we care about are within <pre>..</pre>
   --
@@ -991,16 +1055,72 @@ function cleanupBody(body, cbInfo)
 
   -- Clean up the end of line, and replace HTML tags
   --
+  body = string.gsub(body, "&#9;", "\t")
+  body = string.gsub(body, "&#09;", "\t")
+  body = string.gsub(body, "&#10;", "\n")
+  body = string.gsub(body, "&#13;", "\r")
+  body = string.gsub(body, "&#32;", " ")
+  body = string.gsub(body, "&#33;", "!")
+  body = string.gsub(body, "&#35;", "#")
+  body = string.gsub(body, "&#36;", "$")
+  -- cdmackie: this should be escaped
+  body = string.gsub(body, "&#37;", "%%")
+  body = string.gsub(body, "&#38;", "&")
+  body = string.gsub(body, "&#39;", "'")
+  body = string.gsub(body, "&#40;", "(")
+  body = string.gsub(body, "&#41;", ")")
+  body = string.gsub(body, "&#42;", "*")
+  body = string.gsub(body, "&#43;", "+")
+  body = string.gsub(body, "&#44;", ",")
+  body = string.gsub(body, "&#45;", "-")
+  body = string.gsub(body, "&#46;", ".")
+  body = string.gsub(body, "&#47;", "/")
+  body = string.gsub(body, "&#58;", ":")
+  body = string.gsub(body, "&#59;", ";")
+  body = string.gsub(body, "&#60;", "<")
+
+  -- cdmackie: these mess up QP and b64 encoded attachments
+  --body = string.gsub(body, "&#61;2E", ".")
+  --body = string.gsub(body, "&#61;3D", "=")
+  --body = string.gsub(body, "&#61;20", " ")
+  --body = string.gsub(body, "&#61;09", "\t")
+  --body = string.gsub(body, "&#61;96", "-")
+  --body = string.gsub(body, "&#61;\r\n", "")
+  --body = string.gsub(body, "&#61;92", "'")
+
+  body = string.gsub(body, "&#61;", "=")
+  body = string.gsub(body, "&#62;", ">")
+  body = string.gsub(body, "&#63;", "?")
+  body = string.gsub(body, "&#64;", "@")
+  body = string.gsub(body, "&#91;", "[")
+  body = string.gsub(body, "&#92;", "\\")
+  body = string.gsub(body, "&#93;", "]")
+  body = string.gsub(body, "&#94;", "^")
+  body = string.gsub(body, "&#95;", "_")
+  body = string.gsub(body, "&#96;", "`")
+  body = string.gsub(body, "&#123;", "{")
+  body = string.gsub(body, "&#124;", "|")
+  body = string.gsub(body, "&#125;", "}")
+  body = string.gsub(body, "&#126;", "~")
+  body = string.gsub(body, "&#199;", "")
+
   body = string.gsub(body, "\r", "")
   body = string.gsub(body, "\n", "\r\n")
-  body = string.gsub(body, "&amp;", "&")
-  body = string.gsub(body, "&lt;", "<")
-  body = string.gsub(body, "&gt;", ">")
-  body = string.gsub(body, "&quot;", "\"")
   body = string.gsub(body, "&#34;", "\"")
-  body = string.gsub(body, "&nbsp;", " ")
+  -- cdmackie: these mess up QP attachments in Live
+  -- but still needed for classic
+  if internalState.bLiveGUI == false then
+  	body = string.gsub(body, "&amp;", "&")
+  	body = string.gsub(body, "&lt;", "<")
+  	body = string.gsub(body, "&gt;", ">")
+  	body = string.gsub(body, "&quot;", "\"")
+  	body = string.gsub(body, "&nbsp;", " ")
+  end
   body = string.gsub(body, "<!%-%-%$%$imageserver%-%->", internalState.strImgServer)
 
+  -- cdmackie: POP protocol: lines starting with a dot must be escaped dotdot
+  body = string.gsub(body, "\r\n%.", "\r\n%.%.")
+
   -- We've now at least seen one block, attempt to clean up the headers
   --
   if (cbInfo.bFirstBlock == true) then
@@ -1027,6 +1147,14 @@ function user(pstate, username)
   internalState.strDomain = domain
   internalState.strUser = user
 
+  -- Override the domain variable if it is set in the login parameter
+  --
+  local val = (freepops.MODULE_ARGS or {}).domain or nil
+  if val ~= nil then
+    log.dbg("Hotmail: Using overridden domain: " .. val)
+    internalState.strDomain = val
+  end
+
   -- If the flag emptyTrash is set to 1 ,
   -- the trash will be emptied on 'quit'
   --
@@ -1126,6 +1254,22 @@ function pass(pstate, password)
     -- Execute the function saved in the session
     --
     func()
+
+    -- Should we force a logout.  If this session runs for more than a day, things
+    -- stop working
+    --
+    local currTime = os.clock()
+    local diff = currTime - internalState.loginTime
+    if diff > globals.nSessionTimeout then 
+      logout() 
+      log.dbg("Logout forced to keep hotmail session fresh and tasty!  Yum!\n")
+      log.dbg("Session removed - Account: " .. internalState.strUser .. 
+        "@" .. internalState.strDomain .. "\n")
+      log.raw("Session removed (Forced by Hotmail timer) - Account: " .. internalState.strUser .. 
+        "@" .. internalState.strDomain) 
+      session.remove(hash())
+      return loginHotmail()
+    end
 		
     return POPSERVER_ERR_OK
   else
@@ -1185,7 +1329,8 @@ function quit_update(pstate)
         post = postBase
       end
     elseif internalState.bLiveGUI == true and internalState.bLiveLightGUI and get_mailmessage_flag(pstate, i, MAILMESSAGE_DELETE) then
-      uidlsLight = "SelectedMessages=" .. get_mailmessage_uidl(pstate, i) .. "&"
+      -- cdmackie: uidlsLight should be appended for multiple messages
+      uidlsLight = uidlsLight .. "SelectedMessages=" .. get_mailmessage_uidl(pstate, i) .. "&"
       dcnt = dcnt + 1
     elseif internalState.bLiveGUI == true and get_mailmessage_flag(pstate, i, MAILMESSAGE_DELETE) then
       if i > 1 then
@@ -1220,7 +1365,7 @@ function quit_update(pstate)
     log.dbg("Sending Trash url: " .. cmdUrl .. " - " .. post)
     local body, err = browser:post_uri(cmdUrl, post)
     if (body == nil) then -- M7 Only - DELETE SOON!
-      post = string.format(globals.strCmdDeletePostLiveOls, internalState.strMBox, 
+      post = string.format(globals.strCmdDeletePostLiveOld, internalState.strMBox, 
         internalState.strTrashId, uidls)
       local body, err = browser:post_uri(cmdUrl, post)
     end
@@ -1249,7 +1394,7 @@ function quit_update(pstate)
     end
   elseif internalState.bEmptyTrash and internalState.bLiveGUI then
     cmdUrl = string.format(globals.strCmdEmptyTrashLive, internalState.strMailServer, internalState.strUserId)
-    local post = string.format(globals.strCmdEmptyTrashLivePost, internalState.strTrashId)
+    local post = string.format(globals.strCmdEmptyTrashLivePost, internalState.strTrashId, internalState.strMT)
     log.dbg("Sending Empty Trash URL: " .. cmdUrl .."\n")
     local body, err = browser:post_uri(cmdUrl, post)
     if not body or err then
@@ -1283,14 +1428,7 @@ function quit_update(pstate)
   local currTime = os.clock()
   local diff = currTime - internalState.loginTime
   if diff > globals.nSessionTimeout then 
-    if (internalState.bLiveGUI) then
-      cmdUrl = string.format(globals.strCmdLogoutLive, internalState.strMailServer)
-    else
-      cmdUrl = string.format(globals.strCmdLogout, internalState.strMailServer)
-    end
-    log.dbg("Sending Logout URL: " .. cmdUrl .. "\n")
-    local body, err = getPage(browser, cmdUrl)
- 
+    logout() 
     log.dbg("Logout forced to keep hotmail session fresh and tasty!  Yum!\n")
     log.dbg("Session removed - Account: " .. internalState.strUser .. 
       "@" .. internalState.strDomain .. "\n")
@@ -1311,6 +1449,24 @@ function quit_update(pstate)
   return POPSERVER_ERR_OK
 end
 
+function logout() 
+  local browser = internalState.browser
+  local cmdUrl
+  if (internalState.bLiveGUI) then
+    cmdUrl = string.format(globals.strCmdLogoutLive, internalState.strMailServer)
+  else
+    cmdUrl = string.format(globals.strCmdLogout, internalState.strMailServer)
+  end
+  log.dbg("Sending Logout URL: " .. cmdUrl .. "\n")
+  local body, err = getPage(browser, cmdUrl)
+  if (internalState.bLiveGUI) then
+    cmdUrl = string.match(body, '<meta http-equiv="refresh" content="%d+;url=([^"]+)" />')
+    if (cmdUrl ~= nil) then
+      body, err = getPage(browser, cmdUrl)
+    end
+  end
+end
+
 -- Stat command for the live gui
 --
 function LiveStat(pstate) 
@@ -1460,6 +1616,7 @@ function stat(pstate)
       internalState.strCrumb, internalState.strMBox);
   end
   local baseUrl = cmdUrl
+  local nextPageUrl = nil
 
   -- Keep a list of IDs that we've seen.  With yahoo, their message list can 
   -- show messages that we've already seen.  This, although a bit hacky, will
@@ -1566,6 +1723,10 @@ function stat(pstate)
   local function funcProcessLiveLight(body)
     lastNMsgs = nMsgs
 
+    -- Figure out if there are more pages with messages
+    --
+    nextPageUrl = string.match(body, globals.strMsgListNextPagePatLiveLight)
+
     -- Tokenize out the message ID and size for each item in the list
     --    
     for uidl, size in string.gfind(body, globals.strMsgLiveLightPattern) do
@@ -1624,6 +1785,16 @@ function stat(pstate)
   -- change the command url
   --
   local function funcCheckForMorePages(body) 
+    -- 
+    if (internalState.bLiveLightGUI) then
+      if (nextPageUrl == nil) then
+        return true
+      else
+        cmdUrl = "http://" .. internalState.strMailServer .. "/mail/" .. nextPageUrl
+        return false
+      end		
+    end
+
     -- See if there are messages remaining
     --
     if nMsgs < nTotMsgs then
@@ -1730,7 +1901,7 @@ function stat(pstate)
 
       if internalState.statLimit ~= nil then
         local nMaxMsgs = internalState.statLimit
-        if (nTotMsgs > nMaxMsgs) then
+        if (nTotMsgs == 0 or nTotMsgs > nMaxMsgs) then
           nTotMsgs = nMaxMsgs
         end
       end  
