@@ -17,7 +17,7 @@
 -- ************************************************************************** --
 --  FreePOPs @libero.it, @inwind.it, @blu.it, @iol.it webmail interface
 --  
---  $Id: libero.lua,v 1.57 2007/06/18 12:46:22 gareuselesinge Exp $
+--  $Id: libero.lua,v 1.58 2007/11/10 23:04:49 gareuselesinge Exp $
 --  
 --  Released under the GNU/GPL license
 --  Written by Enrico Tassi <gareuselesinge@users.sourceforge.net>
@@ -446,7 +446,7 @@ function stat(pstate)
 		-- statE and statG
 		-- print(s)
 		local x = mlex.match(s,libero_string.statE,libero_string.statG)
-		 x:print()
+		 --x:print()
 		
 		-- the number of results
 		local n = x:count()
