@@ -1,7 +1,7 @@
 -- ************************************************************************** --
 --  FreePOPs pop3 forwarding plugin
 -- 
---  $Id: popforward.lua,v 1.20 2007/01/14 16:49:37 gareuselesinge Exp $
+--  $Id: popforward.lua,v 1.23 2007/10/28 12:33:13 gareuselesinge Exp $
 -- 
 --  Released under the GNU/GPL license
 --  Written by Enrico Tassi <gareuselesinge@users.sourceforge.net>
@@ -26,8 +26,8 @@ PLUGIN_PARAMETERS = {
 		en = [[If you start the plugin with the username foo@popforward.lua then you need this option to set the real username]],}	
 	},
 	{name = "host", description = {
-		it = [[L'hostname del server POP3 a cui connetterti, puoi anche specificare la porta separandola con :. Esempio: 'in.virgilio.it:110'.]],
-		en = [[The POP3 server hostname. You can specify the port in the hostname:portnumber way.]],}	
+		it = [[L'hostname del server POP3 a cui connetterti, puoi anche specificare la porta separandola con :. Esempio: 'in.virgilio.it:110'.If host is a lua function (you can set that changing the config.lua file but not on the fly) it is called with the username and should return both host and port.]],
+		en = [[The POP3 server hostname. You can specify the port in the hostname:portnumber way. If host is a lua function (you can set that changing the config.lua file but not on the fly) it is called with the username and should return both host and port.]],}	
 	},
 	{name = "port", description = {
 		it = [[Per specificare la porta dell'host a cui connettersi, se non gia' specificato in host con i :]],
@@ -58,15 +58,25 @@ pf_state = {
 	pipe_limit = 0,
 	listed = false,
 	stat_done = false,
+	is_freepops = false,
 }
 
 -- ************************************************************************** --
 --  helpers
 -- ************************************************************************** --
 
+-- this should be kept in sync with the pop3 server
+-- initialized by init()
+freepops_pop3_errors = {}
+
 -- ask for command and call f on the result
 function single_line(cmd,f)
-	local l = pf_state.socket:send(cmd)
+	local l
+	if pf_state.socket ~= nil then
+		l = pf_state.socket:send(cmd)
+	else 
+		l = -1
+	end
 	
 	if l < 0 then 
 		log.error_print("Short send of "..l..
@@ -77,7 +87,11 @@ function single_line(cmd,f)
 	local l = pf_state.socket:recv() or "-ERR network error"
 	if not (string.find(l,"^+OK")) then 
 		log.error_print(l)
-		return POPSERVER_ERR_UNKNOWN 
+		if pf_state.is_freepops then
+			return freepops_pop3_errors[l] or POPSERVER_ERR_UNKNOWN
+		else
+			return POPSERVER_ERR_UNKNOWN 
+		end
 	end
 
 	if f then
@@ -178,7 +192,12 @@ function user(pstate,username)
 		return POPSERVER_ERR_AUTH
 	end
 	
-	local host,port=string.match(freepops.MODULE_ARGS.host,"(.*):(%d+)")
+	local host,port
+	if type(freepops.MODULE_ARGS.host) == "string" then
+		host,port = string.match(freepops.MODULE_ARGS.host,"(.*):(%d+)")
+	elseif type(freepops.MODULE_ARGS.host) == "function" then
+	        host,port = freepops.MODULE_ARGS.host(username)
+	end
 	if host == nil then
 		host = freepops.MODULE_ARGS.host
 	end
@@ -210,6 +229,8 @@ function user(pstate,username)
 		return POPSERVER_ERR_NETWORK
 	end
 	
+	pf_state.is_freepops = string.match(l,'^+OK FreePOPs')
+
 	return single_line("USER "..
 		(freepops.MODULE_ARGS.realusername or username),nil)
 end
@@ -388,6 +409,19 @@ function init(pstate)
 	-- checks on globals
 	freepops.set_sanity_checks()
 
+	-- initilize freepops_pop3_errors 
+	freepops_pop3_errors = {
+        	["-ERR SYNTAX ERROR"] = POPSERVER_ERR_SYNTAX,
+        	["-ERR NETWORK ERROR"] = POPSERVER_ERR_NETWORK,
+        	["-ERR AUTH FAILED"] = POPSERVER_ERR_AUTH,
+        	["-ERR INTERNAL ERROR"] = POPSERVER_ERR_INTERNAL,
+        	["-ERR NO SUCH MESSAGE"] = POPSERVER_ERR_NOMSG,
+        	["-ERR MAILBOX LOCKED"] = POPSERVER_ERR_LOCKED,
+        	["-ERR INTERNAL: END OF STREAM"] = POPSERVER_ERR_EOF,
+        	["-ERR DELAY TIME NOT EXPIRED, RETRY LATER"] = POPSERVER_ERR_TOOFAST,
+        	["-ERR UNKNOWN ERROR, PLEASE FIX"] = POPSERVER_ERR_UNKNOWN
+	}
+
 	return POPSERVER_ERR_OK
 end
 
