@@ -1,14 +1,15 @@
 -- ************************************************************************** --
 --  FreePOPs @virgilio.it, @tin.it webmail interface
 --  
---  $Id: tin.lua,v 1.61 2007/06/22 18:46:42 gareuselesinge Exp $
+--  $Id: tin.lua,v 1.90 2007/11/16 14:13:40 nvhs Exp $
 --  
 --  Released under the GNU/GPL license
 --  Written by Enrico Tassi <gareuselesinge@users.sourceforge.net>
 -- ************************************************************************** --
 
+
 -- these are used in the init function
-PLUGIN_VERSION = "0.2.0"
+PLUGIN_VERSION = "0.2.11e"
 PLUGIN_NAME = "Tin.IT"
 PLUGIN_REQUIRE_VERSION = "0.2.0"
 PLUGIN_LICENSE = "GNU/GPL"
@@ -27,9 +28,23 @@ dovrebbero funzionare. Ecco un esempio di user name per controllare la
 cartella Spam: foo@virgilio.it?folder=Spam]],
 		}	
 	},
+  {name = "limit", description = {
+		it = [[
+    Se si hanno tante e-mail da scaricare, ad esempio a seguito di un lungo periodo
+    in cui non si &egrave; pi&ugrave; scaricata la posta, si potrebbe voler decidere di scaricare
+    un tot di messaggi alla volta, anche per evitare che eventuali errori di comunicazione
+    in fase di scaricamento di tutti i messaggi compromettano la corretta cancellazione
+    degli stessi sul server, con conseguente necessit&agrave; di riscaricarli tutti dall'inizio.
+    Ad esempio, per scaricare un massimo di 100 e-mail alla volta, si pu&ograve; specificare
+    come nome utente: foo@virgilio.it?limit=100 . Da notare che il limite citato &egrave;
+    indicativo e che il numero esatto di e-mail effettivamente scaricate potrebbe
+    essere superiore al valore specificato (dipende dal numero di messaggi per pagina da visualizzare,
+    impostato nelle opzioni della webmail per la propria casella di posta).]],
+		}	
+	},
 }
 PLUGIN_DESCRIPTIONS = {
-	it="Questo plugin vi per mette di leggere le mail che avete "..
+	it="Questo plugin vi permette di leggere le mail che avete "..
 	   "in una mailbox @virgilio.it, @tin.it, @alice.it o @tim.it. "..
 	   "Per usare questo plugin dovete usare il vostro indirizzo email "..
 	   "completo come username e la vostra password reale come password.",
@@ -101,6 +116,12 @@ local tin_string = {
 	body_end = [[</div></td>%s*<td width="5"><spacer type="block" width="5" height="1"></td>%s*<td width="1" bgcolor="#FFFFFF"><spacer type="block" width="1" height="1"></td>%s*</tr>%s*</table>%s*<!%-%-FINE TABELLA LISTING MAIL%-%->]],
 	 attachE = ".*<a.*href='/cp/ps/Mail/ViewAttachment>.*<img>.*</a>",
 	 attachG = "O<X>O<O>X<O>",
+	-- by nvhs for html image
+	 imageE = "<IMG .*/cp/ps/Mail/ViewAttachment.*>",
+	 imageG = "<X>",
+	-- by nvhs for attach  mail
+	 mailE = ".*<a.*href *= *'/cp/ps/Mail/Email>.*<img>.*</a>",
+	 mailG = "O<X>O<O>X<O>",
 	-- The uri to delete some messages
 	--   whearewe(), domain, username, t, s, 
 	delete = "http://%s/cp/ps/Mail/Delete?d=%s&u=%s&t=%s&style=&l=it&s=%s",
@@ -134,6 +155,7 @@ internal_state = {
 	password = nil,
 	b = nil,
 	folder="INBOX",
+	limit=nil,
 	reverse_lookup = {},
 	list_begin = -1
 }
@@ -218,6 +240,14 @@ function aaa_encode(u,p,s)
 	return asc2hex(curl.escape(base64.encode(u.."|"..p.."|"..s.."|")))
 end
 
+function add_webmail_in_front(s)
+	if string.match(s, '^webmail') then
+		return s
+	else
+		return "webmail."..s
+	end
+end
+
 --------------------------------------------------------------------------------
 -- Serialize the internal_state
 --
@@ -417,7 +447,8 @@ function user(pstate,username)
 	internal_state.domain = domain
 	internal_state.name = name
 	internal_state.folder = freepops.MODULE_ARGS.folder or "INBOX"
-	
+	internal_state.limit = tonumber(freepops.MODULE_ARGS.limit) or math.huge
+  
 	return POPSERVER_ERR_OK
 end
 
@@ -484,7 +515,7 @@ function quit_update(pstate)
 
 	-- shorten names, not really important
 	local b = internal_state.b
-	local popserver = b:wherearewe()
+	local popserver = add_webmail_in_front(b:wherearewe())
 	local session_id_s = internal_state.session_id_s
 	local session_id_t = internal_state.session_id_t
 	local domain = internal_state.domain
@@ -536,7 +567,7 @@ function stat(pstate)
 	local session_id_s = internal_state.session_id_s
 	local session_id_t = internal_state.session_id_t
 	local b = internal_state.b
-	local popserver = b:wherearewe()
+	local popserver = add_webmail_in_front(b:wherearewe())
 	local domain = internal_state.domain
 	local user = internal_state.name
 	local pop_login = user .. "@" .. domain
@@ -545,7 +576,7 @@ function stat(pstate)
 	-- the check_f function, see later
 	local page = 1
 	local uri = string.format(tin_string.first,
-		b:wherearewe(), internal_state.folder,
+		popserver, internal_state.folder,
 		domain, user, session_id_t, session_id_s, page)
 	
 	-- The action for do_until
@@ -615,7 +646,11 @@ function stat(pstate)
 			domain, user, session_id_t, session_id_s, page)
 		return false
 	end
-	local function check_f (s) 
+	local function check_f (s)
+		if get_popstate_nummesg(pstate) >= internal_state.limit then 
+			-- if a limit was set, stop
+			return true 
+		end
 		local tmp = string.find(s,tin_string.no_next)
 		if tmp ~= nil and count == 0 then
 			return next_page()
@@ -733,7 +768,7 @@ end
 -- popserver_callback to send the data
 -- 
 function tin_parse_webmessage(wherearewe, data)
-	local head, body, body_html, attach = nil, nil, nil, {}
+	local head, body, body_html, attach ,inlineids = nil, nil, nil, {} , {}
 
 	-- extract headers 
 	local headersE = ".*<script>.*var *hd *=</script>.*<br>.*<br>.*</div>"
@@ -752,12 +787,11 @@ function tin_parse_webmessage(wherearewe, data)
 	head = string.gsub(head, "&gt;", ">")
 	head = string.gsub(head, "\r([^\n])", 
 		function(capture) return "\r\n" .. capture end)
-	head = string.gsub(head, "à", "")
-	head = string.gsub(head, "è", "")
-	head = string.gsub(head, "é", "")
-	head = string.gsub(head, "ì", "")
-	head = string.gsub(head, "ò", "")
-	head = string.gsub(head, "ù", "")
+	head = string.gsub(head, "([^\r])\n","%1\r\n")
+	-- patch by galdom
+	head = string.gsub(head, "\\&#39;", "'")
+	head = string.gsub(head, "&amp;", "&")
+
 	
 	-- locate body
 	local _, begin_body = string.find(data, tin_string.body_start)
@@ -768,45 +802,78 @@ function tin_parse_webmessage(wherearewe, data)
 		"[Cc][Oo][Nn][Tt][Ee][Nn][Tt]%-[Tt][Yy][Pp][Ee]%s*:%s*"..
 		"[Tt][Ee][Xx][Tt]/[Pp][Ll][Aa][Ii][Nn]")
 	if found == nil then
+		body_html = string.sub(data, begin_body + 1, end_body - 1)
 		head = mimer.remove_lines_in_proper_mail_header(head,
 			{"content%-type"})
-		body_html = string.sub(data, begin_body + 1, end_body - 1)
 	else
 		body = string.sub(data, begin_body + 1, end_body - 1)
-                body = string.gsub(body, "^%s+", "")
-                body = string.gsub(body, "%s+$", "")
+		body = string.gsub(body, "^%s+", "")
+            body = string.gsub(body, "%s+$", "")
 		body = string.gsub(body, "<br/>", "\r\n");
 		body = string.gsub(body, "<a href[^>]*>", "");
 		body = string.gsub(body, "</a>", "");
 		body = string.gsub(body, "&lt;", "<")
 		body = string.gsub(body, "&gt;", ">")
-                body = string.gsub(body, "&quot;", "\"")
-                body = string.gsub(body, "&#39;", "'")
-                body = string.gsub(body, "&amp;", "&")
-		body = string.gsub(body, "à", "")
-		body = string.gsub(body, "è", "")
-		body = string.gsub(body, "é", "")
-		body = string.gsub(body, "ì", "")
-		body = string.gsub(body, "ò", "")
-		body = string.gsub(body, "ù", "")
+		body = string.gsub(body, "&quot;", "\"")
+            body = string.gsub(body, "&#39;", "'")
+            body = string.gsub(body, "&amp;", "&")
 	end
 	
+	
+	-- Added to nvhs to delete scroll bar 
+	if not (body_html == nil) then 
+		body_html  = string.gsub(body_html, "auto;width:570px;height", "auto;height")
+		body_html  ="<html>  <body> "..body_html.."</body> </html>"
+	end
+
 	-- extract attachments
 	local x = mlex.match(data, tin_string.attachE, tin_string.attachG) 
-	x:print()
+	--x:print()
 	for i = 1, x:count() do
 		local url = x:get(0,i-1)
 		local name = x:get(1,i-1)
-
 		local name = string.match(name, "^(.*) %(")
-
-		local url = string.match(url, 
+		local url = string.match(url,
 			"href%s*=%s*'(/cp/ps/Mail/ViewAttachment[^']*)'")
-		
 		attach[name] = "http://"..wherearewe..url
 	end
 	
-	return head, body, body_html, attach
+	-- by nvhs extract attach mail
+	
+	local x = mlex.match(data, tin_string.mailE, tin_string.mailG) 
+	--x:print()
+	for i = 1, x:count() do
+		local url = x:get(0,i-1)
+		local name = x:get(1,i-1)
+		local name = string.match(name,"%s*%p*(.*)")
+		name=name..".html"
+		local url = string.match(url,
+			"href%s*=%s*'(/cp/ps/Mail/Email[^']*)'")
+		attach[name] = "http://"..wherearewe..url
+
+	end
+
+	-- by nvhs for html image
+	
+	local y = mlex.match(data, tin_string.imageE, tin_string.imageG)
+	-- y:print()
+	for i = 1, y:count() do
+		local url = y:get(0,i-1)
+		url = string.match(url,
+			"/cp/ps/Mail/ViewAttachment.-&id=%d*")
+		if url ~= nil then 
+			attach[url] = "http://"..wherearewe..url
+			inlineids[url]=url
+		end
+		
+	end
+	-- replace url with cid
+	if body_html ~= nil then
+		body_html  = string.gsub(body_html,
+			"/cp/ps/Mail/ViewAttachment.-&id=%d*","cid:%1")	
+	end
+	
+	return head, body, body_html, attach ,inlineids
 end
 
 function is_a_list_needed(msg, uidl)
@@ -834,7 +901,7 @@ function retr(pstate,msg,data)
 	local session_id_t = internal_state.session_id_t
 	local session_id_s = internal_state.session_id_s
 	local b = internal_state.b
-	local popserver = b:wherearewe()
+	local popserver = add_webmail_in_front(b:wherearewe())
 	local domain = internal_state.domain
 	local user = internal_state.name
 	local pop_login = user .. "@" .. domain
@@ -850,22 +917,26 @@ function retr(pstate,msg,data)
 	if relist then
 		-- we do a list that begins with our message
 		local uri_l = string.format(tin_string.first,
-			b:wherearewe(), internal_state.folder,
+			popserver, internal_state.folder,
 			domain, user, session_id_t, session_id_s, msg)
 		local _,_ = b:get_uri(uri_l)
 		internal_state.list_begin = msg
 	end
 	-- whearewe, mailbox, username, username, uidl, t, s
-	local uri = string.format(tin_string.save,b:wherearewe(),
+	local uri = string.format(tin_string.save,popserver,
 		folder, domain, user, user, uidl, session_id_t, session_id_s,sl)
 	
 	-- tell the browser to fetch
-	local head,f,rc = b:get_head_and_body(uri)
+	local head,f = b:get_head_and_body(uri)
+	if head == nil then
+		log.error_print("Error fetching "..uri..": ".. (f or 'nil'))
+		return POPSERVER_ERR_UNKNOWN
+	end
 	local found,_,ctype = string.find(head,
 		"[Cc][Oo][Nn][Tt][Ee][Nn][Tt]%-[Tt][Yy][Pp][Ee]%s*:"..
 		"%s*[^;\r]+;%s*[Cc][Hh][Aa][Rr][Ss][Ee][Tt]=\"?([^\"\r]*)")
 	if found == nil then
-		ctype = "iso-8859-1"
+		ctype = "UTF-8"
 	end
 	
 	if f == nil then
@@ -880,11 +951,12 @@ function retr(pstate,msg,data)
 		--b:show()
 		return POPSERVER_ERR_UNKNOWN
 	end
-
-	local wherearewe = b:wherearewe()
-	local head,body,body_html,attach = tin_parse_webmessage(wherearewe, f)
+	
+	local wherearewe = add_webmail_in_front(b:wherearewe())
+	local head,body,body_html,attach,inlineids = tin_parse_webmessage(wherearewe, f)
 	local cb = mimer.callback_mangler(common.retr_cb(data))
-	mimer.pipe_msg(head,body,body_html,"http://"..wherearewe,attach,b,cb,nil,ctype)
+	head = string.gsub(head,"([Cc][Hh][Aa][Rr][Ss][Ee][Tt]=).-[;\n]","%1\""..ctype.."\"; ")
+	mimer.pipe_msg(head,body,body_html,"http://"..wherearewe,attach,b,cb,inlineids,ctype)
 		
 	return POPSERVER_ERR_OK
 end
@@ -901,7 +973,7 @@ function top(pstate,msg,lines,data)
 	local session_id_t = internal_state.session_id_t
 	local session_id_s = internal_state.session_id_s
 	local b = internal_state.b
-	local popserver = b:wherearewe()
+	local popserver = add_webmail_in_front(b:wherearewe())
 	local domain = internal_state.domain
 	local user = internal_state.name
 	local pop_login = user .. "@" .. domain
@@ -917,22 +989,26 @@ function top(pstate,msg,lines,data)
 	if relist then
 		-- we do a list that begins with our message
 		local uri_l = string.format(tin_string.first,
-			b:wherearewe(), internal_state.folder,
+			popserver, internal_state.folder,
 			domain, user, session_id_t, session_id_s, msg)
 		local _,_ = b:get_uri(uri_l)
 		internal_state.list_begin = msg
 	end
 	-- whearewe, mailbox, username, username, uidl, t, s
-	local uri = string.format(tin_string.save,b:wherearewe(),
+	local uri = string.format(tin_string.save,popserver,
 		folder, domain, user, user, uidl, session_id_t, session_id_s,sl)
 	
 	-- tell the browser to fetch
-	local head,f,rc = b:get_head_and_body(uri)
+	local head,f = b:get_head_and_body(uri)
+	if head == nil then
+		log.error_print("Error fetching "..uri..": ".. (f or 'nil'))
+		return POPSERVER_ERR_UNKNOWN
+	end
 	local found,_,ctype = string.find(head,
 		"[Cc][Oo][Nn][Tt][Ee][Nn][Tt]%-[Tt][Yy][Pp][Ee]%s*:"..
 		"%s*[^;\r]+;%s*[Cc][Hh][Aa][Rr][Ss][Ee][Tt]=\"?([^\"\r]*)")
 	if found == nil then
-		ctype = "iso-8859-1"
+		ctype = "UTF-8"
 	end
 	
 	if f == nil then
@@ -941,10 +1017,11 @@ function top(pstate,msg,lines,data)
 		return POPSERVER_ERR_NETWORK
 	end
 
-	local wherearewe = b:wherearewe()
-	local head,body,body_html,attach = tin_parse_webmessage(wherearewe, f)
+	local wherearewe = add_webmail_in_front(b:wherearewe())
+	local head,body,body_html,attach,inlineids = tin_parse_webmessage(wherearewe, f)
 	local global = common.new_global_for_top(lines,nil)
 	local cb = mimer.callback_mangler(common.top_cb(global,data,true))
+	head = string.gsub(head,"([Cc][Hh][Aa][Rr][Ss][Ee][Tt]=).-[;\n]","%1\""..ctype.."\"; ")
 	mimer.pipe_msg(head,body,body_html,"http://"..wherearewe,attach,b,cb,nil,ctype)
 		
 	return POPSERVER_ERR_OK
