@@ -6,11 +6,12 @@
 --  yahoo.it added by Nicola Cocchiaro <ncocchiaro@users.sourceforge.net>
 --  yahoo.ie added by Bruce Williamson <aztrix@yahoo.com>
 --  Contributions by Przemyslaw Wroblewski <przemyslaw.wroblewski@gmail.com>
+--  Contributions from Kevin Edwards
 -- ************************************************************************** --
 
 -- Globals
 --
-PLUGIN_VERSION = "0.1.9j"
+PLUGIN_VERSION = "0.1.9p"
 PLUGIN_NAME = "yahoo.com"
 PLUGIN_REQUIRE_VERSION = "0.2.0"
 PLUGIN_LICENSE = "GNU/GPL"
@@ -21,7 +22,7 @@ PLUGIN_AUTHORS_CONTACTS =
 	{"russells (at) despammed (.) com",
          "ncocchiaro (at) users (.) sourceforge (.) net"}
 PLUGIN_DOMAINS = {"@yahoo.com", "@yahoo.ie", "@yahoo.it", "@yahoo.ca", "@rocketmail.com", "@yahoo.com.ar",
-                  "@yahoo.co.in", "@yahoo.com.tw", "@yahoo.co.uk", "@yahoo.com.cn",
+                  "@yahoo.co.in", "@yahoo.co.id", "@yahoo.com.tw", "@yahoo.co.uk", "@yahoo.com.cn",
                   "@yahoo.es", "@yahoo.de", "@talk21.com", "@btinternet.com", "@yahoo.com.au",
 }
 
@@ -187,8 +188,8 @@ local globals = {
 
   -- Command URLS
   --
-  strCmdMsgList = "%sym/ShowFolder?box=%s&Npos=%d&Nview=%s&order=up&sort=date&reset=1&Norder=up",
-  strCmdMsgView = "%sym/ShowLetter?box=%s&PRINT=1&Nhead=f&toc=1&MsgId=%s&bodyPart=%s",
+  strCmdMsgList = "%sym/ShowFolder?box=%s&Npos=%d&Nview=%s&view=%s&order=up&sort=date&reset=1&Norder=up",
+  strCmdMsgView = "%sya/download?box=%s&PRINT=1&Nhead=f&MsgId=%s&bodyPart=%s",
   strCmdMsgWebView = "%sym/ShowLetter?box=%s&MsgId=%s",
   strCmdEmptyTrash = "%sym/ShowFolder?ET=1&", 
   strCmdEmptyBulk = "%sym/ShowFolder?EB=1&", 
@@ -222,6 +223,7 @@ local globals = {
   strYahooCn = "cn",  -- China
   strYahooHk = "hk",  -- Hong Kong
   strYahooIn = "in",  -- India
+  strYahooId = "id",  -- Indonesia
   strYahooJp = "jp",  -- Japan
   strYahooKr = "kr",  -- Korea
   strYahooSg = "sg",  -- Singapore
@@ -313,6 +315,7 @@ internalState = {
 -- Set to true to enable Raw Logging
 --
 local ENABLE_LOGRAW = false
+local LOGRAW_DBG_FIRST = true
 
 -- The platform dependent End Of Line string
 -- e.g. this should be changed to "\n" under UNIX, etc.
@@ -322,6 +325,10 @@ local EOL = "\r\n"
 --
 log = log or {} -- fast hack to make the xml generator happy
 log.raw = function ( line, data )
+  if LOGRAW_DBG_FIRST then
+    log.dbg( line )
+  end
+  
   if not ENABLE_LOGRAW then
     return
   end
@@ -369,6 +376,29 @@ function fixCRLF( data )
   return str
 end
 
+-- Convert any object to a string
+--
+function tostr( val )
+  local val_type = type(val)
+  if val_type == "table" and getmetatable(val) == nil then
+    local tmp = {}
+    table.foreach(val,function (k,v)
+      local s = k .. "=" .. tostr(v)
+      table.insert(tmp,s)
+    end)
+    return "{" .. table.concat(tmp) .. "}"
+  elseif val_type == "number" then
+    return string.format("%d",val)
+  elseif val_type == "string" then
+    return string.format("%q",val)
+  elseif val_type == "boolean" then
+    return tostring(val)
+  elseif val == nil then
+    return "nil"
+  end
+  return val_type .. " : " .. tostring(val)
+end
+
 -- ************************************************************************** --
 --  Helper functions
 -- ************************************************************************** --
@@ -405,7 +435,7 @@ function checkForNewGUI(browser, body)
   local server = string.match(browser:whathaveweread(), globals.strRegExpMailServerNew)
   if (server ~= nil) then 
     log.dbg("Detected New Version of the Yahoo interface!\n")
-    log.raw("Detected New Version of the Yahoo interface!\n")
+    log.raw("Detected New Version of the Yahoo interface!\n", browser:whathaveweread())
     log.dbg("Yahoo Mail Server: " .. server .. "\n")
 
     internalState.strMailServer = server
@@ -425,12 +455,16 @@ function loginYahoo()
   log.raw( "Entering loginYahoo()" )
 
   if internalState.loginDone then
+    log.raw( "internalState.loginDone" )
     return POPSERVER_ERR_OK
   end
 
   -- Create a browser to do the dirty work (It must be set to IE 6.0)
   --
   internalState.browser = browser.new("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; {02ABE9F9-C33D-95EA-0C84-0B70CD0AC3F8}; .NET CLR 1.1.4322)")
+--  internalState.browser = browser.new("Mozilla/4.0 (compatible; U; en)")
+--  internalState.browser = browser.new("Mozilla/3.0 (U; en)")
+--  internalState.browser = browser.new("FreePOPs/2.5 (U; en)")
   local SSLEnabled = browser.ssl_enabled()
 
   -- Define some local variables
@@ -469,7 +503,9 @@ function loginYahoo()
 
   -- Login to Yahoo
   --
+  log.raw( "login get: " .. globals.strLoginPage )
   local body, err = browser:get_uri(globals.strLoginPage)
+  log.raw( "login response: err=" .. tostr(err), body )
   
   if body ~= nil then
     challengeCode = string.match(body, globals.strLoginChallenge)
@@ -488,19 +524,37 @@ function loginYahoo()
   else -- if we didn't get the challenge code, then login in cleartext
     post = string.format(globals.strLoginPostData, intFlag, username, password, uVal)
   end
+
+  log.raw( "login challenge post: \nurl=" .. url .. " \npost=" .. post )
   body, err = browser:post_uri(url, post)
+  log.raw( "login challenge response: err=" .. tostr(err), body )
 
   -- Check for redirect
   --
   local str = string.match(body, globals.strRedirectNew)
   if (str ~= nil) then
+    log.raw( "login redirect get: " .. str )
     body, err = browser:get_uri(str)
+    log.raw( "login redirect response: err=" .. tostr(err), body )
   end
 
-  local bNewGui = checkForNewGUI(browser, body)
-
-  if (bNewGui == true) then
-    return loginNewYahoo(browser, body) 
+  -- Check for NewGUI and Try Beta
+  local bNewGui = false
+  local url = browser:whathaveweread()
+  log.raw("browser:whathaveweread()=" .. url)
+  str = string.match(url, '/dc/try_mail')
+  if (str == nil) then
+    -- it's not a "try" page.  Check if its the new gui.
+    bNewGui = checkForNewGUI(browser, body)
+    if (bNewGui == true) then
+      return loginNewYahoo(browser, body) 
+    end
+  else
+    -- it is a try page, so reply, no thanks
+    post = "newStatus=1"
+    log.raw( "try beta post: \nurl=" .. url .. " \npost=" .. post )
+    body, err = browser:post_uri(url, post)
+    log.raw( "try beta response: err=" .. tostr(err), body )
   end
 
   -- Do some error checking
@@ -508,12 +562,14 @@ function loginYahoo()
   if (body == nil) then
     -- No connection
     --
+    log.raw( "Login Failed: body == nil" )
     log.error_print("Login Failed: Unable to make connection")
     return POPSERVER_ERR_NETWORK
   end
 
   local str = string.match(body, '<input type="text" id="secword" name=".secword"')
   if (str ~= nil) then
+    log.raw("Login Failed: Yahoo is using an image verification.")
     log.error_print("Login Failed: Yahoo is using an image verification.  Please login through the web.")
     return POPSERVER_ERR_NETWORK
   end
@@ -522,9 +578,7 @@ function loginYahoo()
   -- 
   local str = string.match(body, globals.strRetLoginBadPassword)
   if str ~= nil then
-    log.raw("------ Returned Page saying invalid password ------\n")
-    log.raw(body)
-    log.raw("------ End Page saying invalid password -------\n")
+    log.raw("Returned Page saying invalid password: ", body)
     log.error_print("Login Failed: Invalid Password")
     return POPSERVER_ERR_AUTH
   end
@@ -534,10 +588,11 @@ function loginYahoo()
   local str = string.match(body, globals.strRegExpMailServer)
   if str == nil then
     log.error_print("Login Failed: Unable to find mail server")
-    log.raw("Unable to find mail server: " .. body)
+    log.raw("Login Failed: Unable to find mail server url in body.")
     return POPSERVER_ERR_UNKNOWN
   else
     internalState.strMailServer = str
+    log.raw("internalState.strMailServer = " .. str)
   end
 
   -- If we are using HTTPS, we need to look for the meta-refresh link
@@ -545,7 +600,9 @@ function loginYahoo()
   if (challengeCode ~= nil) then
     str = string.match(body, globals.strRegExpMetarefresh)
     if (str ~= nil) then
+      log.raw("HTTPS meta-refresh get: " .. str)
       body, err = browser:get_uri(str)
+      log.raw("HTTPS meta-refresh response: err=" .. tostr(err), body)
       
       -- Here's the other check (SSL) for the beta interface
       --
@@ -566,7 +623,7 @@ function loginYahoo()
 	
   -- Debug info
   --
-  log.dbg("Created session for " .. 
+  log.raw("Created session for " .. 
     internalState.strUser .. "@" .. internalState.strDomain .. "\n")
 
   -- Return Success
@@ -582,8 +639,7 @@ function loginNewYahoo(browser, body)
   local str = string.match(body, globals.strRegExpCrumbNew)
   if str == nil then
     log.error_print("Yahoo - unable to parse out crumb value.  Deletion will fail.")
-    log.raw("Yahoo - unable to parse out crumb value.  Deletion will fail, Body: " .. 
-      body)
+    log.raw("Yahoo - unable to parse out crumb value.  Deletion will fail, Body: ", body)
     return POPSERVER_ERR_UNKNOWN
   end
   log.dbg("Crumb Value: " .. str)
@@ -706,6 +762,12 @@ function downloadYahooMsg(pstate, msg, nLines, data)
   --
   local cb = downloadMsg_cb(cbInfo, data)
 
+  -- Remove the SMTP envelope From_ line (Yahoo's mbox format) if it's first
+  --
+  if string.sub(headers,1,5) == "From " then
+    headers = string.gsub(headers, "From .-\n", "", 1);
+  end
+
   -- Remove the quote-printed encoding line from the header
   --
   headers = string.gsub(headers, "Content%-Transfer%-Encoding: quoted%-printable%s+", "");
@@ -760,13 +822,13 @@ function downloadYahooMsg(pstate, msg, nLines, data)
 
   -- Do we need to mark the message as unread?
   --
-  if internalState.bMarkMsgAsUnread == true then
-    if internalState.bNewGUI then
-      log.dbg("Marking as message: " .. uidl .. " as unread");
-      markMsgUnread(uidl)
-    else
-      local cmdUrl = string.format(globals.strCmdMsgWebView, internalState.strMailServer,
-        internalState.strMBox, msgid);
+  if internalState.bNewGUI and internalState.bMarkMsgAsUnread == true then
+    log.dbg("Marking as message: " .. uidl .. " as unread");
+    markMsgUnread(uidl)
+  elseif internalState.bNewGUI == false then
+    local cmdUrl = string.format(globals.strCmdMsgWebView, internalState.strMailServer,
+      internalState.strMBox, msgid);
+    if (internalState.bMarkMsgAsUnread) then
       local str, _ = browser:get_uri(cmdUrl) 
       str = string.match(str, globals.strMsgMarkUnreadPat)
       if str == nil then
@@ -776,6 +838,10 @@ function downloadYahooMsg(pstate, msg, nLines, data)
         log.dbg("Marking as message: " .. msgid .. " as unread, url: " .. cmdUrl);
         browser:get_uri(cmdUrl) -- We don't care about the results.
       end
+    else
+      -- Mark the message as read
+      --
+      local str, _ = browser:get_head(cmdUrl) 
     end
   end
 
@@ -1198,6 +1264,8 @@ function user(pstate, username)
     internalState.strIntFlag = globals.strYahooIt
   elseif domain == "yahoo.ca" then
     internalState.strIntFlag = globals.strYahooCa
+  elseif domain == "yahoo.co.id" then
+    internalState.strIntFlag = globals.strYahooId
   elseif domain == "yahoo.co.in" then
     internalState.strIntFlag = globals.strYahooIn
   elseif domain == "yahoo.fr" then
@@ -1349,7 +1417,7 @@ function pass(pstate, password)
     local func, err = loadstring(sessID)
     if not func then
       log.error_print("Unable to load saved session (Account: " ..
-        internalState.strUser .. "@" .. internalState.strDomain .. "): ".. err)
+        internalState.strUser .. "@" .. internalState.strDomain .. "): ".. tostr(err))
       return loginYahoo()
     end
 		
@@ -1532,7 +1600,7 @@ function stat(pstate)
   local nPage = 0
   local nMsgs = 0
   local cmdUrl = string.format(globals.strCmdMsgList, internalState.strMailServer,
-    internalState.strMBox, nPage, internalState.strView);
+    internalState.strMBox, nPage, internalState.strView, internalState.strView);
 
   -- Keep a list of IDs that we've seen.  With yahoo, their message list can 
   -- show messages that we've already seen.  This, although a bit hacky, will
@@ -1694,7 +1762,7 @@ function stat(pstate)
       --
       browser = internalState.browser
       cmdUrl = string.format(globals.strCmdMsgList, internalState.strMailServer,
-        internalState.strMBox, nPage, internalState.strView);
+        internalState.strMBox, nPage, internalState.strView, internalState.strView);
 
       -- Retry to load the page
       --
