@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: luabox.c,v 1.9 2007/01/13 22:36:25 gareuselesinge Exp $
+ * $Id: luabox.c,v 1.10 2007/10/22 21:43:48 gareuselesinge Exp $
  * This file is part of FreePOPs (http://www.freepops.org)					*
  * This file is distributed under the terms of GNU GPL license.			   *
  ******************************************************************************/
@@ -35,6 +35,7 @@
 #include "crypto_lua.h"
 #include "lfs.h"
 #include "dpipe_lua.h"
+#include "stats_lua.h"
 
 #include "log.h"
 #define LOG_ZONE "LUABOX"
@@ -55,6 +56,7 @@ static struct entry_t libs[LUABOX_LAST] = {
 		{"crypto",luaopen_crypto},
 		{"lfs",luaopen_lfs},
 		{"dpipe",luaopen_dpipe},
+		{"stats",luaopen_stats},
 };
 
 lua_State* luabox_genbox(unsigned long intial_stuff){
