@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: luabox.h,v 1.6 2007/01/13 22:36:25 gareuselesinge Exp $
+ * $Id: luabox.h,v 1.7 2007/10/22 21:43:48 gareuselesinge Exp $
  * This file is part of FreePOPs (http://www.freepops.org)                    *
  * This file is distributed under the terms of GNU GPL license.               *
  ******************************************************************************/
@@ -32,16 +32,17 @@
 #define LUABOX_CRYPTO		(1<<11)
 #define LUABOX_LUAFILESYSTEM	(1<<12)
 #define LUABOX_DPIPE		(1<<13)
+#define LUABOX_STATS		(1<<14)
 
 #define LUABOX_FREEPOPS	(LUABOX_POP3SERVER|LUABOX_MLEX|LUABOX_STRINGHACK|\
 			 LUABOX_SESSION|LUABOX_CURL|LUABOX_PSOCK|\
 			 LUABOX_BASE64|LUABOX_GETDATE|LUABOX_REGULAREXP|\
 			 LUABOX_LXP|LUABOX_LOG|LUABOX_CRYPTO|\
-			 LUABOX_LUAFILESYSTEM|LUABOX_DPIPE)
+			 LUABOX_LUAFILESYSTEM|LUABOX_DPIPE|LUABOX_STATS)
 
 #define LUABOX_FULL (LUABOX_FREEPOPS)
 
-#define LUABOX_LAST 14
+#define LUABOX_LAST 15
 
 //! generates a luaVM loading initial_stuff libraries
 lua_State* luabox_genbox(unsigned long intial_stuff);
